//***************************************************************************
// MEMDC.C     (c) Microsoft.
//
// WARNING: ANY USE BY YOU OF THIS CODE IS AT YOUR OWN RISK. 
// Microsoft provides this code "as is" without warranty of any
// kind, either express or implied, including but not limited to the implied
// warranties of merchantability and/or fitness for a particular purpose.
//
/****************************************************************************

    PROGRAM: MemDC.c

    PURPOSE: Demonstrate the use of offscreen bitmaps

    FUNCTIONS:

	WinMain()         - calls initialization function, processes message loop
	InitApplication() - initializes window data and registers window
	InitInstance()    - saves instance handle and creates main window
	WndProc()         - process messages for main window
	OnCommand()       - handle WM_COMMAND messages
	AboutDlgProc()    - window proc for the "About" dialog
	DrawOnDC()        - does the complex drawing on a DC
	DrawFromMemory()  - BitBlts the memory bitmap to the screen

    HISTORY: 5/22/95  - wrote it - jmh

****************************************************************************/
#include <Windows.H>
#include "Resource.h"


// ======================================================================
// Function Prototypes
//
int PASCAL WinMain(HANDLE, HANDLE, LPSTR, int);
BOOL InitApplication(HANDLE);
BOOL InitInstance(HANDLE, int);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT OnCommand(HWND, WPARAM, LPARAM);
BOOL CALLBACK AboutDlgProc(HWND, UINT, WPARAM, LPARAM);
void DrawOnDC(HDC, RECT);
void DrawFromMemory(HDC, RECT);
// ======================================================================


// ======================================================================
// Global Variables
HINSTANCE	hInst = NULL;
HWND		hWndMain = NULL;
char		szAppName[] = "MemDC";
char		szTitle[] = "Prueba bajo Windows.";
BOOL		bUseMemoryBitmap = TRUE;
HBITMAP		hBitmap = NULL;
// ======================================================================


// ======================================================================
/*
  FUNCTION   : WinMain

  PARAMETERS :	HANDLE - this instance
				HANDLE - previous instance
				LPSTR  - command line
				int    - requested display state

  PURPOSE    : calls initialization function, processes message loop

  RETURNS    : int

  COMMENTS   : Windows recognizes this function by name as the initial entry
               point for the program.  This function calls the application
               initialization routine, if no other instance of the program is
               running, and always calls the instance initialization routine.
               It then executes a message retrieval and dispatch loop that is
               the top-level control structure for the remainder of execution.
               The loop is terminated when a WM_QUIT message is received, at
               which time this function exits the application instance by
               returning the value passed by PostQuitMessage().
               If this function must abort before entering the message loop,
               it returns the value FALSE.

  HISTORY    : 5/22/95 - Created
*/
int PASCAL WinMain(HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow)
{
	MSG msg;

	// Is there a previous instance?
	if( ! hPrevInstance )
	{
		// If not, do the one-time initializations now
		if( ! InitApplication( hInstance ) )
		{
			return FALSE;
		}
	}
	// Now initialize this instance
	if( ! InitInstance( hInstance, nCmdShow ) )
	{
		return FALSE;
	}
	// The main message loop for this app
	while( GetMessage( &msg, NULL, 0, 0 ) )
	{
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}
	return msg.wParam;
}
// End WinMain
// ======================================================================



// ======================================================================
/*
  FUNCTION   : InitApplication

  PARAMETERS : HANDLE hInstance - Handle to the current instance

  PURPOSE    : Initializes window data and registers window class

  RETURNS    : BOOL - TRUE is Success, FALSE is failure

  COMMENTS   : This function is called at initialization time only if no
               other instances of the application are running.  This function
               performs initialization tasks that can be done once for any
               number of running instances.
               In this case, we initialize a window class by filling out a
               data structure of type WNDCLASS and calling the Windows
               RegisterClass() function.  Since all instances of this
               application use the same window class, we only need to do this
               when the first instance is initialized.

  HISTORY    : 5/22/95 - Created
*/
BOOL InitApplication( HANDLE hInstance )
{
	WNDCLASS wc;

	// Fill out a WNDCLASS struct for our main window class
	wc.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wc.lpfnWndProc   = (WNDPROC)WndProc;
	wc.cbClsExtra    = 0;
	wc.cbWndExtra    = 0;
	wc.hInstance     = hInstance;
	wc.hIcon         = LoadIcon( hInstance, MAKEINTRESOURCE(MEMDC_ICON) );
	wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	wc.lpszMenuName  = MAKEINTRESOURCE(MEMDC_MENU);
	wc.lpszClassName = szAppName;

	// Register the main window class
	return RegisterClass( &wc );
}
// End InitApplication
// ======================================================================



// ======================================================================
/*
  FUNCTION   : InitInstance

  PARAMETERS : HANDLE  hInstance - Current instance identifier
               int     nCmdShow  - Param for first ShowWindow() call

  PURPOSE    : Saves instance handle and creates main window

  RETURNS    : BOOL - TRUE is Success, FALSE is failure

  COMMENTS   : This function is called at initialization time for every
               instance of this application.  This function performs
               initialization tasks that cannot be shared by multiple
               instances.
               In this case, we save the instance handle in a static variable
               and create and display the main program window.

  HISTORY    : 5/22/95 - Created
*/
BOOL InitInstance( HANDLE hInstance, int nCmdShow )
{
	//  Store the current instance handle
	hInst = hInstance;

	// Create the main window
	hWndMain = CreateWindow(
		szAppName,
		szTitle,
		WS_OVERLAPPED | WS_CAPTION |
		WS_SYSMENU | WS_MINIMIZEBOX | WS_MAXIMIZEBOX,
		CW_USEDEFAULT, 0,
		CW_USEDEFAULT, 0,
		NULL,
		NULL,
		hInstance,
		NULL );

	if( hWndMain == NULL )
	{	// If we failed to create the window, return failure
		return FALSE;
	}

	// Display the window
	ShowWindow( hWndMain, nCmdShow );
	UpdateWindow( hWndMain );

	return TRUE;
}
// End InitInstance
// ======================================================================



// ======================================================================
/*
  FUNCTION   : WndProc

  PARAMETERS : HWND hWnd        -  window handle
               UINT message     -  type of message
               WPARAM wParam    -  additional information
               LPARAM lParam    -  additional information

  PURPOSE    : Processes WM_ style messages for the main window.

  RETURNS    : LRESULT

  COMMENTS   : 
  
  HISTORY    : 5/22/95 - Created
*/
LRESULT CALLBACK WndProc( HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam )
{
	switch( Message )
	{
		case WM_PAINT:
		{	// Windows has determined that we need to paint
			PAINTSTRUCT		ps;
			RECT			Rect;

			BeginPaint( hWnd, &ps );

			// How big is our window?
			GetClientRect( hWnd, &Rect );
		
			if( bUseMemoryBitmap )
			{
				// We are drawing using a memory bitmap
				// BitBlt it to the screen
				DrawFromMemory( ps.hdc, Rect );
			}
			else
			{
				// We are not using a memory bitmap
				// Draw directly on the screen DC
				DrawOnDC( ps.hdc, Rect );
			}
			EndPaint( hWnd, &ps );
		}
		break; // End WM_PAINT

		case WM_CREATE:
		{	// Our window is getting created, now would be a
			// good time to create the memory bitmap
			#define WIDTH(x)	(x.right-x.left+1)	// Macro to get rect width
			#define HEIGHT(x)	(x.bottom-x.top+1)	// Macro to get rect height
			RECT	Rect;
			HBITMAP	hOldBitmap;
			HDC		hDC, hMemDC;

			// How big is the window?
			GetClientRect( hWnd, &Rect );
			// Need a DC
			hDC = GetDC( hWnd );
			// And a memory DC
			hMemDC = CreateCompatibleDC( hDC );
			// Create a bitmap big enough to hold the window's image
			hBitmap = CreateCompatibleBitmap( hDC, WIDTH(Rect), HEIGHT(Rect) );
			// Select the bitmap into the memory DC
			hOldBitmap = SelectObject( hMemDC, hBitmap );
			// draw our nifty pattern on it
			DrawOnDC( hMemDC, Rect );
			// Select the original bitmap back in
			SelectObject( hMemDC, hOldBitmap );
			// Clean up
			DeleteDC( hMemDC );
			ReleaseDC( hWnd, hDC );
			#undef WIDTH
			#undef HEIGHT
		}
		break; // End WM_CREATE
		
		case WM_INITMENU:
		{	// This message allows us to appropriately check/uncheck
			// the menu option for using a memory bitmap
			HMENU  hMenu =  (HMENU)wParam;

			CheckMenuItem( hMenu, ID_OPTIONS_USEMEMORYBITMAP, 
							MF_BYCOMMAND | 
							(bUseMemoryBitmap?MF_CHECKED:MF_UNCHECKED) );
		}
		break; // End WM_INITMENU

		case WM_COMMAND:
			// Handle WM_COMMAND message (menu items, etc)
			return OnCommand( hWnd, wParam, lParam );
		break; // End WM_COMMAND

		case WM_CLOSE:
			// Time to say 'goodbye' <g>
			if( hBitmap != NULL )
				DeleteObject( hBitmap );
			DestroyWindow( hWnd );
			PostQuitMessage( 0 );
		break; // End WM_CLOSE

		default:
			// Pass unknown messages on to Windows.
			return DefWindowProc( hWnd, Message, wParam, lParam );
		break; // End default
	}
	return 0l;
}
// End WndProc
// ======================================================================



// ======================================================================
/*
  FUNCTION   : OnCommand

  PARAMETERS : HWND hWnd        -  window handle
               WPARAM wParam    -  additional information
               LPARAM lParam    -  additional information

  PURPOSE    : Processes WM_COMMAND messages for the main window.

  RETURNS    : LRESULT

  COMMENTS   : This function processes the WM_COMMAND messages for
               the main window. This includes the menu item messages.

  HISTORY    : 5/22/95 - Created
*/
LRESULT OnCommand( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	switch( LOWORD( wParam ) )
	{
		// Menu option for flag for using memory bitmap
		case ID_OPTIONS_USEMEMORYBITMAP:
			bUseMemoryBitmap = ! bUseMemoryBitmap;
		break;

		// Shut 'er down!
		case ID_F_EXIT:
			PostMessage( hWnd, WM_CLOSE, (WPARAM)0, (LPARAM)0 );
		break;

		// User would like to see the AboutBox
		case ID_HELP_ABOUT:
			DialogBox( hInst, MAKEINTRESOURCE(MEMDC_ABOUT_DLG), hWnd, AboutDlgProc );
		break;

		// What the heck message is this anyway?
		default:
			MessageBox( hWnd, "Unknown WM_COMMAND", "Error", MB_OK );
		break;
	}
	return 0l;
}
// End OnCommand
// ======================================================================


// ======================================================================
/*
  FUNCTION   : AboutDlgProc

  PARAMETERS : HWND hWnd        -  dialog window handle
               UINT message     -  type of message
               WPARAM wParam    -  additional information
               LPARAM lParam    -  additional information

  PURPOSE    : Processes messages for the About Dialog Box

  RETURNS    : LRESULT

  COMMENTS   : 
  
  HISTORY    : 5/22/95 - Created
*/
BOOL CALLBACK AboutDlgProc( HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam )
{
	switch( Msg )
	{
		case WM_INITDIALOG:
		break;

		case WM_CLOSE:
			PostMessage( hWnd, WM_COMMAND, IDCANCEL, 0l );
		break;

		case WM_COMMAND:
			switch( LOWORD(wParam) )
			{
				case IDOK:
					EndDialog( hWnd, TRUE );
				break;

				case IDCANCEL:
					EndDialog( hWnd, FALSE );
				break;

			}
		break;
		default:
			return FALSE;
		break;
	}
	return TRUE;
}
// End AboutDlgProc
// ======================================================================



// ======================================================================
/*
  FUNCTION   : DrawOnDC

  PARAMETERS : HDC hDC          -  device context handle
               RECT Rect        -  the rectangle in which to draw

  PURPOSE    : Does our drawing on the specified DC

  RETURNS    : void

  COMMENTS   : This is the drawing code. It is called once to
               draw on the memory bitmap, and also during each
			   paint message if we are not using the memory
			   bitmap.
  
  HISTORY    : 5/22/95 - Created
*/
void DrawOnDC( HDC hDC, RECT Rect )
{
	LONG	x1, y1, x2, y2, Index = 0;
	HPEN	hPen, hOldPen;

	
	// Since LineTo doesn't include the endpoint...
	Rect.top--;
	Rect.right++;
	// Draw from top to bottom
	for( x1=Rect.left,x2 = Rect.right,y1 = Rect.top,y2 = Rect.bottom;
			y1 < Rect.bottom; y1++, y2-- )
	{
		Index++;
		hPen = CreatePen( PS_SOLID, 0, PALETTEINDEX(Index%10) );
		hOldPen = SelectObject( hDC, hPen );
		MoveToEx( hDC, x1, y1, NULL );
		LineTo( hDC, x2, y2 );
		SelectObject( hDC, hOldPen );
		DeleteObject( hPen );
	}
	// Draw from left to right
	for( ; x1 < Rect.right; x1++, x2-- )
	{
		Index++;
		hPen = CreatePen( PS_SOLID, 0, PALETTEINDEX(Index%10) );
		hOldPen = SelectObject( hDC, hPen );
		MoveToEx( hDC, x1, y1, NULL );
		LineTo( hDC, x2, y2 );
		SelectObject( hDC, hOldPen );
		DeleteObject( hPen );
	}
}
// End DrawOnDC
// ======================================================================


// ======================================================================
/*
  FUNCTION   : DrawFromMemory

  PARAMETERS : HDC hDC          -  device context handle
               RECT Rect        -  the rectangle in which to draw

  PURPOSE    : Blt's the memory bitmap to the DC

  RETURNS    : void

  COMMENTS   : This is the code used to paint the client
               area when 'Use memory bitmap' is on. It
			   simply BitBlt's the memory bitmap to the DC.
  
  HISTORY    : 5/22/95 - Created
*/
void DrawFromMemory( HDC hDC, RECT Rect )
{
	HDC 		hMemDC;
	HBITMAP		hOldBitmap;

	// Need a memory DC
	hMemDC = CreateCompatibleDC( hDC );
	// Select in the bitmap
	hOldBitmap = SelectObject( hMemDC, hBitmap );
	// Blt it
	BitBlt( hDC, Rect.left, Rect.top, Rect.right - Rect.left, 
			Rect.bottom - Rect.top, hMemDC, 0, 0, SRCCOPY );
	// Clean up
	SelectObject( hMemDC, hOldBitmap );
	DeleteDC( hMemDC );
}
// End DrawFromMemory
// ======================================================================


